﻿'use strict';

function ChatGuestItem(userProviderKey, currentDate, chatHub, versatileArg, chatProfile) {
    var ci = this;
    if (Array.isArray(versatileArg)) {
        ChatItem.call(this, userProviderKey, currentDate, chatHub, versatileArg);
    }
    else {
        ci.versatileArg = versatileArg;
        ChatItem.call(this, userProviderKey, currentDate, chatHub, null);
    }

    if (chatProfile)
        ci.chatProfile = chatProfile;
    ci.secludedGuestUserSession = ko.observable(null);
    ci.guestHistory = ko.observable(null);
    ci.guestUserSession = ko.pureComputed(ci.getGuestUserSession, ci);
    ci.guestInitials = ko.pureComputed(ci.getGuestInitials, ci);
    ci.waitingDate = ko.pureComputed(ci.getWaitingDate, ci);
    ci.humanizedWaitingDuration = ko.pureComputed(ci.getHumanizedWaitingDuration, ci);
    ci.firstAcceptanceDate = ko.pureComputed(ci.getFirstAcceptanceDate, ci);
    ci.orderDate = ko.pureComputed(ci.getOrderDate, ci);
    ci.callStartDate = ko.pureComputed(ci.getCallStartDate, ci);
    ci.chatProfileDate = ko.pureComputed(ci.getChatProfileDate, ci);
    ci.firstAcceptanceDuration = ko.pureComputed(ci.getFirstAcceptanceDuration, ci);
    ci.callDuration = ko.pureComputed(ci.getCallDuration, ci);
}

ChatGuestItem.prototype = Object.create(ChatItem.prototype);

ChatGuestItem.MAX_DATE = new Date(8640000000000000);
ChatGuestItem.MIN_DATE = new Date(-8640000000000000);

ChatGuestItem.toTwoDigitNumber = function (number) {
    return ("0" + number).slice(-2);
};

ChatGuestItem.prototype.preInit = function () {
    var ci = this;

    if (typeof ci.versatileArg === "undefined") {
        ci.__proto__.__proto__.preInit.call(ci);
    }
    else {
        if (ci.versatileArg instanceof ChatSessionViewModel) {
            ci.sessionId = ci.versatileArg.sessionId;
            ci.chatProfile = ci.versatileArg.chatProfile;
        }
        else if ('Session' in ci.versatileArg) {
            ci.sessionId = ci.versatileArg.Session.SessionId;
        }
        else throw "Argument exception";
    }
};

ChatGuestItem.prototype.init = function () {
    var ci = this;
    if (typeof ci.versatileArg === "undefined") { // sesna je na serveru a mam od ni userSession (operator only)
        return ci.__proto__.__proto__.init.call(ci);
    }
    else return new Promise(function (resolve, reject) {
        if ('Session' in ci.versatileArg) { //jedina guestUserSession (literal object ze serveru) - sesna je na serveru a nemam od ni userSession (operator only)
            ci.secludedGuestUserSession(ci.versatileArg);
            ci.viewModel(null);
            resolve();
        }
        else if (ci.versatileArg instanceof ChatSessionViewModel) { // inicializacni viewModel hosta (guest only)
            ci.viewModel(ci.versatileArg);
            resolve();
        }
        else
            reject();

        delete ci.versatileArg;
    });
};

ChatGuestItem.prototype.getGuestInitials = function () {
    var ci = this;
    let guestUserSession = ci.guestUserSession();
    if (guestUserSession) {
        let initials = guestUserSession.User.FullName.match(/\b\w/g) || [];
        initials = ((initials.shift() || '') + (initials.pop() || '')).toUpperCase();
        return initials;
    }
};

ChatGuestItem.prototype.getGuestUserSession = function () {
    var ci = this;
    var secludedGuestUserSession = ci.secludedGuestUserSession();
    return secludedGuestUserSession ? secludedGuestUserSession : ci.viewModel().guestUserSession();
};

ChatGuestItem.prototype.getSession = function () {
    var ci = this;
    let guestUserSession = ci.guestUserSession();
    return guestUserSession ? guestUserSession.Session : null;
};

ChatGuestItem.prototype.getMyUserSession = function () {
    var ci = this;
    var viewModel = ci.viewModel();
    return viewModel ? viewModel.myUserSession() : null;
};

ChatGuestItem.prototype.getGuestHistoryIfNeeded = function () {
    var ci = this;
    if (!ci.guestHistory()) {
        $.getJSON(RQ_URL + "/Chat/GetGuestHistory/" + ci.guestUserSession().User.UserProviderKey).done(function (res) {
            if (!res) {
                console.warn("No guest history result.");
                return;
            }

            ci.guestHistory(res);
        }).fail(function (error) {
            console.warn(error);
        });
    }
};

ChatGuestItem.prototype._getMyExpression = function () {
    var ci = this;
    let session = ci.session();
    if (session) {
        let expression = session.Expressions[ci.userProviderKey];
        if (expression)
            return expression;
    }

    return null;
};

ChatGuestItem.prototype.getWaitingDate = function () {
    var ci = this;
    let session = ci.session();
    if (session) {
        let waitingMessage = session.WaitingMessage;
        if (waitingMessage) {
            return new Date(waitingMessage.Date);
        }
    }

    var expression = ci._getMyExpression();
    if (expression !== null && expression.Attitude === null && expression.Offer) {
        return new Date(expression.Offer.Time);
    }
};

ChatGuestItem.prototype.getHumanizedWaitingDuration = function () {
    var ci = this;
    let waitingDate = ci.waitingDate();
    if (waitingDate && ci.currentDate()) {
        return moment.duration(ci.currentDate() - waitingDate).humanize();
    }
};

ChatGuestItem.prototype.getOrderDate = function () {
    var ci = this;
    let session = ci.session();
    if (session) {
        let waitingMessage = session.WaitingMessage;
        if (waitingMessage) {
            return new Date(waitingMessage.Date);
        }
    }

    return ci.firstAcceptanceDate();
};

ChatGuestItem.prototype.getCallStartDate = function () {
    var ci = this;
    let session = ci.session();
    if (session && session.CallStartTime) {
        return new Date(session.CallStartTime);
    }

    return null;
};

ChatGuestItem.prototype.getChatProfileDate = function () {
    var ci = this;
    let session = ci.session();
    if (session) {
        let waitingMessage = session.WaitingMessage;
        if (waitingMessage) {
            return new Date(waitingMessage.Date);
        }

        for (var expressionProviderKey in session.Expressions) {
            if (expressionProviderKey === ci.userProviderKey && session.Expressions[expressionProviderKey].Attitude === true) {
                return ChatGuestItem.MIN_DATE;
            }
        }
    }
};

ChatGuestItem.prototype.getFirstAcceptanceDuration = function () {
    var ci = this;
    let firstAcceptanceDate = ci.firstAcceptanceDate();
    let compareDate = ci.session().ClosingDateUtc ? new Date(ci.session().ClosingDateUtc) : ci.currentDate();

    if (firstAcceptanceDate && compareDate) {
        let duration = moment.duration(compareDate - firstAcceptanceDate);
        return "{0}:{1}:{2}".format(ChatGuestItem.toTwoDigitNumber(duration.hours()), ChatGuestItem.toTwoDigitNumber(duration.minutes()), ChatGuestItem.toTwoDigitNumber(duration.seconds()));
    }
};

ChatGuestItem.prototype.getCallDuration = function () {
    var ci = this;
    let callStartDate = ci.callStartDate();
    if (callStartDate) {
        let duration = moment.duration(ci.currentDate() - callStartDate);
        return "{0}:{1}:{2}".format(ChatGuestItem.toTwoDigitNumber(duration.hours()), ChatGuestItem.toTwoDigitNumber(duration.minutes()), ChatGuestItem.toTwoDigitNumber(duration.seconds()));
    }

    return null;
};

ChatGuestItem.prototype.getFirstAcceptanceDate = function () {
    var ci = this;
    let minAttitudeTime = ChatGuestItem.MAX_DATE;
    let session = ci.session();
    if (session) {
        let expressions = ci.session().Expressions;
        for (var expressionName in expressions) {
            let expression = expressions[expressionName];
            if (expression.Attitude === true) {
                let attitudeTime = new Date(expression.AttitudeTime);
                if (attitudeTime < minAttitudeTime) {
                    minAttitudeTime = attitudeTime;
                }
            }
        }
    }

    return minAttitudeTime !== ChatGuestItem.MAX_DATE ? minAttitudeTime : null;
};

ChatGuestItem.prototype.createAndInitChatSessionViewModel = function (userSessions) {
    var ci = this;
    return new Promise(function (resolve, reject) {
        if (userSessions && ci.secludedGuestUserSession()) {
            var guestUserSessionFound = false;
            for (var i = 0; i < userSessions.length; i++) {
                var userSession = userSessions[i];
                if (userSession.Session.GuestProviderKey === userSession.User.UserProviderKey) {
                    guestUserSessionFound = true;
                    break;
                }
            }

            if (!guestUserSessionFound) {
                userSessions.push(ci.secludedGuestUserSession());
            }
        }

        var viewModel = new ChatSessionViewModel(ci.userProviderKey, ci.sessionId, null, null, ci.chatProfile);
        viewModel.init(userSessions, ci.chatHub).then(function () {
            ci.viewModel(viewModel);
            ci.secludedGuestUserSession(null);
            resolve();
        }).catch(function (error) {
            reject(error);
        });
    });
};

ChatGuestItem.prototype.createTicketRelationsAndRefuse = function (ticketIds) {
    var ci = this;
    return new Promise(function (resolve, reject) {
        ci.chatHub.server.createTicketRelationsOfChatSessionAndRefuse(ci.sessionId, ticketIds).done(function () {
            resolve();
        }).fail(function (error) {
            reject(error);
        });
    });
};