﻿'use strict';

var callMainboard = {};

$(function () {
    var viewModel;
    var callItemUniqueIdSelection;
    var callItemsSubscriptions = {};

    function subscribeServiceWorkerMessage(viewModel) {
        navigator.serviceWorker.addEventListener('message', function (event) {
            switch (event.data.action) {
                case "answer":
                    answerAndOpenCall(event.data.uniqueId);
                    break;
                case "reject":
                    viewModel.rejectCall();
                    break;
                case "hangup":
                    viewModel.hangupCall();
                    break;
            }
        });
    }

    function subscribeCallItemsMutation(viewModel) {
        viewModel.callItems.subscribe(function (changes) {
            changes.forEach(function (change) {
                let callItem = change.value;
                if (change.status === 'added' && callItem.state() === callSummary.callStateEnum.RINGING) {
                    if (callItemUniqueIdSelection === callItem.uniqueId)
                        callItemsSubscriptions[callItem.uniqueId].dispose();
                    else if (callItemUniqueIdSelection !== undefined)
                        displayCallSessionWebNotification(callItem);

                    callItemsSubscriptions[callItem.uniqueId] = subscribeCallItemStateChange(callItem);
                }
                else if (change.status === 'deleted') {
                    if (callItemUniqueIdSelection === callItem.uniqueId)
                        callItemUniqueIdSelection = null;
                    else if (callItemsSubscriptions[callItem.uniqueId]) {
                        deleteCallItem(callItem);
                    }
                }
            });

        }, null, "arrayChange");
    }

    function subscribeSelectedCallItemChanged(viewModel) {
        viewModel.selectedCallItemUniqueId.extend({ notify: "always" }).subscribe(function (newSelectedCallItemUniqueId) {
            callItemUniqueIdSelection = typeof callDashboard !== 'undefined' ? newSelectedCallItemUniqueId : null;
        });
    }

    function subscribeCallItemStateChange(callItem) {
        return callItem.state.subscribe(function (newState) {
            if (newState !== callSummary.callStateEnum.RINGING) {
                deleteCallItem(callItem);
            }
        });
    }

    function deleteCallItem(callItem) {
        callItemsSubscriptions[callItem.uniqueId].dispose();
        delete callItemsSubscriptions[callItem.uniqueId];
        closeCallSessionsWebNotifications(callItem);
    }

    function subscribeViewModel(newViewModel) {
        viewModel = newViewModel;
        if (newViewModel.isNewCallWebNotificationAvailable) {
            subscribeServiceWorkerMessage(newViewModel);
            subscribeSelectedCallItemChanged(newViewModel);
            subscribeCallItemsMutation(newViewModel);
        }
    }

    function openCallItem(uniqueId) {
        if ('callDashboard' in window) {
            viewModel.selectCallItem(uniqueId);
        }
        else {
            window.location.href = '{0}?UniqueId={1}'.format(RQ_URL + '/Call', uniqueId);
        }
    }

    function answerAndOpenCall(uniqueId) {
        viewModel.answerCall().then(function () {
            openCallItem(uniqueId);
        });
    }

    function displayCallSessionWebNotification(callItem) {
        if ("Notification" in window && Notification.permission === "granted") {
            navigator.serviceWorker.ready.then(function (registration) {
                let faviconLink = document.querySelector("link[rel*='icon']");
                let incoming = callItem.way === "In";
                registration.showNotification(incoming ? "Příchozí hovor" : "Odchozí hovor", {
                    body: callItem.selectedUser() ? callItem.selectedUser().FullName + "\n" + (incoming ? callItem.from : callItem.to) : ('(' + (incoming ? callItem.from : callItem.to) + ')'),
                    icon: faviconLink.href,
                    badge: faviconLink.href,
                    requireInteraction: true,
                    timestamp: new Date(callItem.RingingDateUtc),
                    data: {
                        uniqueId: callItem.uniqueId
                    },
                    tag: callItem.uniqueId,
                    silent: true,
                    renotify: true,
                    actions: incoming ? [{
                        action: 'answer',
                        title: 'Přijmout',
                        icon: RQ_URL + '/img/icon-up.png'
                    }, {
                        action: 'reject',
                        title: 'Odmítnout',
                        icon: RQ_URL + '/img/icon-down.png'
                    }] : [{
                        action: 'hangup',
                        title: 'Zavěsit',
                        icon: RQ_URL + '/img/icon-down.png'
                    }]
                });
            }).catch(function (ex) {
                console.error(ex);
            });
        }
    }

    function closeCallSessionsWebNotifications(callItem) {
        if ("Notification" in window && Notification.permission === "granted") {
            navigator.serviceWorker.ready.then(function (registration) {
                registration.getNotifications({tag: callItem.uniqueId}).then(function (notifications) {
                    notifications.forEach(function (notification) {
                        notification.close();
                    });
                })
            });
        }
    }

    function getSubscribedViewModel() {
        return viewModel;
    }

    function initCallPanelDialog() {
        $("#dialog-callPanel").dialog({
            autoOpen: false,
            resizable: false,
            width: "auto",
            height: "auto",
            modal: true,
            buttons: {
                'Zavřít': function () {
                    $(this).dialog("close");
                }
            }
        });
    }

    function openCallPanelDialog() {
        $("#dialog-callPanel").dialog('open');
    }

    function initDOM() {
        ko.bindingHandlers.bootstrapSwitch = {
            init: function (element, valueAccessor, allBindingsAccessor) {
                //initialize bootstrapSwitch
                $(element).bootstrapSwitch(/*allBindingsAccessor().bootstrapSwitchOptions*//*{
                    onSwitchChange: function (e, state) {
                        var clickFunc = allBindingsAccessor().click;
                        if (typeof clickFunc === 'function')
                            clickFunc(ko.dataFor(this));
                    }
                }*/);

                // setting initial value
                $(element).bootstrapSwitch('state', valueAccessor()());

                //handle the field changing
                $(element).on('switchChange.bootstrapSwitch', function (event, state) {
                    var observable = valueAccessor();
                    observable(state);
                });

                // Adding component options
                var options = allBindingsAccessor().bootstrapSwitchOptions || {};
                for (var property in options) {
                    $(element).bootstrapSwitch(property, ko.utils.unwrapObservable(options[property]));
                }

                //handle disposal (if KO removes by the template binding)
                ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                    $(element).bootstrapSwitch("destroy");
                });

            },
            //update the control when the view model changes
            update: function (element, valueAccessor, allBindingsAccessor) {
                var value = ko.utils.unwrapObservable(valueAccessor());

                // Adding component options
                var options = allBindingsAccessor().bootstrapSwitchOptions || {};
                for (var property in options) {
                    $(element).bootstrapSwitch(property, ko.utils.unwrapObservable(options[property]));
                }

                $(element).bootstrapSwitch("state", value);
            }
        };


        $("#callSum").click(function (e) {
            openCallPanelDialog();
        });

        $("#callPauses").on("select2:select", function (e) {
            viewModel.selectPause();
        });
    }

    function pauseTemplate(data) {
        return $('<div></div>', {
            "class": "vertical-center-container",
            html: [
            $('<div></div>', {
                "class": "color-indication",
                css: { "background": data.id !== callSummary.NO_PAUSE_ID ? "red" : "green", "margin": "2px;" }
            }),
            $('<span></span>', {
                css: { "padding": "0px 6px" },
                text: data.id !== callSummary.NO_PAUSE_ID ? data.text : 'Žádná pauza'
            })]
        });
    }

    //ready
    initDOM();
    initCallPanelDialog();
    callMainboard.subscribeViewModel = subscribeViewModel;
    callMainboard.getSubscribedViewModel = getSubscribedViewModel;
    callMainboard.pauseTemplate = pauseTemplate;
    callMainboard.openCallItem = openCallItem;
    callMainboard.answerAndOpenCall = answerAndOpenCall;
});